// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/Texture.hpp"
#include "display/shader/ShaderStructure.hpp"

namespace Dawn {
  struct IShaderData {
    protected:
      void *data = nullptr;
      bool_t dirty = true;

    public:
      std::shared_ptr<ShaderStructure> structure;

      /**
       * Constructs the ShaderData object
       * 
       * @param structure The structure to base this data on.
       */
      IShaderData(
        const std::shared_ptr<ShaderStructure> &structure
      );

      /**
       * Gets the value out of the data by the offset.
       * 
       * @param offset The offset to get the value from.
       * @return The value at the offset.
       */
      template<typename T>
      T get(const size_t offset);

      /**
       * Gets the value out of the data by the offset.
       * 
       * @param offset The offset to get the value from.
       * @return The value at the offset.
       */
      void setRaw(const size_t offset, const void *value, const size_t size);

      /**
       * Sets the value in the data by the offset.
       * 
       * @param offset The offset to set the value at.
       * @param value The value to set at the offset.
       */
      template<typename T>
      void set(const size_t offset, const T value);

      /**
       * Binds the buffer to the shader.
       */
      virtual void bind() = 0;

      /**
       * Uploads the data to the GPU.
       */
      virtual void upload() = 0;

      /**
       * Cleans up the data.
       */
      virtual ~IShaderData();
  };
}